/* controlpointuiaction
 * $AUTHORS$, Tue Jun 6 2006
 *
 */
#include "controlpointuiaction.h"

ControlPointUIAction::ControlPointUIAction(std::vector<ControlPoint>* controlPoints)
:	m_controlPoints(controlPoints),
	m_index(-1)
{
}


ControlPointUIAction::~ControlPointUIAction()
{
}


// return true if a point is selected
bool ControlPointUIAction::selectControlPoint(float x, float y, float minDist)
{
	double minSqDist = minDist*minDist;
	for(unsigned int i=0; i<m_controlPoints->size(); ++i)
	{
		// if position is very close to a control point, return true
		double sqDist = sqDistance(i, x, y);
		if(sqDist<minSqDist)
		{
			m_index = i;
			return true;
		}
	}
	m_index = -1; // invalid index, no selection
	return false; // if not close to a control point, return false
}


void ControlPointUIAction::addControlPoint(float x, float y)
{
	m_controlPoints->push_back(ControlPoint(x,y,0,0));
	m_index = m_controlPoints->size()-1; // current selection is on new added point
}


void ControlPointUIAction::motion(float pos[0])
{
	if(m_index<0) return; // do nothing if no selection
	
	// Control point displacement from previous position
	float dx = pos[0] - (*m_controlPoints)[m_index].x;
	float dy = pos[1] - (*m_controlPoints)[m_index].y;
	
	// Update total displacement
	(*m_controlPoints)[m_index].dx += dx;
	(*m_controlPoints)[m_index].dy += dy;
	
	// Update current position
	(*m_controlPoints)[m_index].x = pos[0];
	(*m_controlPoints)[m_index].y = pos[1];
}


double ControlPointUIAction::sqDistance(int index, float x, float y) const
{
	double diff[2] = { x-(*m_controlPoints)[index].x, y-(*m_controlPoints)[index].y };
	return (diff[0]*diff[0] + diff[1]*diff[1]);
}
