/* deformimagewithcontrolpoint
 * $AUTHORS$, Tue Jun 6 2006
 *
 */
#include "deformimagewithcontrolpoint.h"

DeformImageWithControlPoint::DeformImageWithControlPoint(const Image* image, const std::vector<ControlPoint>* controlPoints)
:	m_image(image),
	m_controlPoints(controlPoints)
{
	m_dx = new ThinPlateSpline(100,100);
	m_dy = new ThinPlateSpline(100,100);
}


DeformImageWithControlPoint::~DeformImageWithControlPoint()
{
	delete m_dx;
	delete m_dy;
}


const int* DeformImageWithControlPoint::getSize() const
{
	return m_image->getSize();
}


unsigned char DeformImageWithControlPoint::get(int i, int j) const
{
	// See where the point (i,j) should be
	const int* imageSize = m_image->getSize();
	float dx = m_dx->get((float)i/imageSize[0],(float)j/imageSize[1]);
	float dy = m_dy->get((float)i/imageSize[0],(float)j/imageSize[1]);
		
	// Pixel intensity
	return m_image->get((int)(i-dx),(int)(j-dy));
}


void DeformImageWithControlPoint::update()
{
	// Needs at least 3 control points
	if(m_controlPoints->size()<3) return;
	
	// Build control point list from grid
	const int* imageSize = m_image->getSize();
	typedef ThinPlateSpline::ControlPoint Point;
	std::vector<Point> controlPoints[2]; // [0] dx, [1] dy
	
	std::vector<ControlPoint>::const_iterator pb = m_controlPoints->begin();
	std::vector<ControlPoint>::const_iterator pe = m_controlPoints->end();
	
	while(pb != pe)
	{
		// Position of control point
		float x = pb->x;
		float y = pb->y;
		
		// Displacement from original position
		float dx = pb->dx;
		float dy = pb->dy;
		
		// Control point position is between 0 and 1
		controlPoints[0].push_back(Point(x/imageSize[0],y/imageSize[1],dx));
		controlPoints[1].push_back(Point(x/imageSize[0],y/imageSize[1],dy));
		
		// next control point
		++pb;
	}
	
	m_dx->setControlPoints(controlPoints[0]);
	m_dy->setControlPoints(controlPoints[1]);
}
