/* deformimagewithgrid
 * $AUTHORS$, Tue Jun 6 2006
 *
 */
#include "deformimagewithgrid.h"

DeformImageWithGrid::DeformImageWithGrid(const Image* image, const Grid* grid)
:	m_image(image),
	m_grid(grid)
{
	m_dx = new ThinPlateSpline(100,100);
	m_dy = new ThinPlateSpline(100,100);
}


DeformImageWithGrid::~DeformImageWithGrid()
{
	delete m_dx;
	delete m_dy;
}


const int* DeformImageWithGrid::getSize() const
{
	return m_image->getSize();
}


unsigned char DeformImageWithGrid::get(int i, int j) const
{
	// See where the point (i,j) should be
	const int* imageSize = m_image->getSize();
	float dx = m_dx->get((float)i/imageSize[0],(float)j/imageSize[1]);
	float dy = m_dy->get((float)i/imageSize[0],(float)j/imageSize[1]);
		
	// Pixel intensity
	return m_image->get((int)(i-dx),(int)(j-dy));
}


void DeformImageWithGrid::update()
{
	// Build control point list from grid
	const int* imageSize = m_image->getSize();
	typedef ThinPlateSpline::ControlPoint Point;
	std::vector<Point> controlPoints[2]; // [0] dx, [1] dy
	for(int j=0; j<m_grid->getSize()[1]; ++j)
		for(int i=0; i<m_grid->getSize()[0]; ++i)
	{
		float x = m_grid->getPointPosition(i,j)[0];
		float y = m_grid->getPointPosition(i,j)[1];
		float dx = x - i*(float)(imageSize[0])/(m_grid->getSize()[0]-1);
		float dy = y - j*(float)(imageSize[1])/(m_grid->getSize()[1]-1);
		
		// Control point position is between 0 and 1
		controlPoints[0].push_back(Point(x/imageSize[0],y/imageSize[1],dx));
		controlPoints[1].push_back(Point(x/imageSize[0],y/imageSize[1],dy));
	}
	
	m_dx->setControlPoints(controlPoints[0]);
	m_dy->setControlPoints(controlPoints[1]);
}
