/* displaygrid
 * $AUTHORS$, Tue Jun 6 2006
 *
 */
 
#include "displaygrid.h"
#include <GL/gl.h>

DisplayGrid::DisplayGrid(Grid* grid)
:	m_grid(grid)
{
}

DisplayGrid::~DisplayGrid()
{
}

void DisplayGrid::display()
{
	// Display the control points
	for(int j=0; j<m_grid->getSize()[1]; ++j)
		for(int i=0; i<m_grid->getSize()[0]; ++i)
	{
		glPushMatrix();
		
		// Display a square where there is a control point
		glTranslatef(m_grid->getPointPosition(i,j)[0], m_grid->getPointPosition(i,j)[1],0);
		glScalef(10,10,1);
		glColor3f(1,1,1);
		glBegin(GL_QUADS);
			glVertex3f(-0.5,-0.5,0);
			glVertex3f(-0.5,+0.5,0);
			glVertex3f(+0.5,+0.5,0);
			glVertex3f(+0.5,-0.5,0);
		glEnd();
		
		glPopMatrix();
	}
	
	// Display connectivity of each control point
	for(int j=0; j<m_grid->getSize()[1]-1; ++j)
		for(int i=0; i<m_grid->getSize()[0]-1; ++i)
	{
		// posA (i,j) ----- posB (i+1,j)
		//     |
		//     |
		// posC (i,j+1)
		float posA[2] = { m_grid->getPointPosition(i,j)[0], m_grid->getPointPosition(i,j)[1] };
		float posB[2] = { m_grid->getPointPosition(i+1,j)[0], m_grid->getPointPosition(i+1,j)[1] };
		float posC[2] = { m_grid->getPointPosition(i,j+1)[0], m_grid->getPointPosition(i,j+1)[1] };
		
		glColor3f(1,1,1);
		glBegin(GL_LINES);
			glVertex3f(posA[0], posA[1], 0);
			glVertex3f(posB[0], posB[1], 0);
			glVertex3f(posA[0], posA[1], 0);
			glVertex3f(posC[0], posC[1], 0);
		glEnd();
	}
	
	// Display connectivity at the border of the grid
	for(int i=0, j=m_grid->getSize()[1]-1; i<m_grid->getSize()[0]-1; ++i)
	{
		float posA[2] = { m_grid->getPointPosition(i,j)[0], m_grid->getPointPosition(i,j)[1] };
		float posB[2] = { m_grid->getPointPosition(i+1,j)[0], m_grid->getPointPosition(i+1,j)[1] };
		
		glColor3f(1,1,1);
		glBegin(GL_LINES);
		glVertex3f(posA[0], posA[1], 0);
		glVertex3f(posB[0], posB[1], 0);
		glEnd();
	}
	for(int j=0, i=m_grid->getSize()[0]-1; j<m_grid->getSize()[1]-1; ++j)
	{
		float posA[2] = { m_grid->getPointPosition(i,j)[0], m_grid->getPointPosition(i,j)[1] };
		float posC[2] = { m_grid->getPointPosition(i,j+1)[0], m_grid->getPointPosition(i,j+1)[1] };
		
		glColor3f(1,1,1);
		glBegin(GL_LINES);
		glVertex3f(posA[0], posA[1], 0);
		glVertex3f(posC[0], posC[1], 0);
		glEnd();
	}
}
