/* grid
 * $AUTHORS$, Tue Jun 6 2006
 *
 */
#include "grid.h"

Grid::Grid(int sizeX, int sizeY)
{
	// Allocate the control points
	m_size[0] = sizeX;
	m_size[1] = sizeY;
	m_points = new Point* [m_size[1]];
	for(int j=0; j<m_size[1]; ++j)
		m_points[j] = new Point [m_size[0]];
}


Grid::~Grid()
{
	// Desallocate the control points
	if(m_points)
	{
		for(int j=0; j<m_size[1]; ++j)
			delete [] m_points[j];
		delete [] m_points;
	}
}

void Grid::initOrthogonal(float spacing[2])
{
	// Initialize the grid as an orthogonal,
	// with a space of -spacing- between each control points
	for(int j=0; j<m_size[1]; ++j)
		for(int i=0; i<m_size[0]; ++i)
	{
		m_points[j][i].pos[0] = i*spacing[0];
		m_points[j][i].pos[1] = j*spacing[1];
	}
}

const float* Grid::getPointPosition(int i, int j) const
{
	// Return the control point position
	if(i<0 || i>=m_size[0] || j<0 || j>=m_size[1]) return 0;
	return m_points[j][i].pos;
}


void Grid::setPointPosition(int i, int j, float pos[2])
{
	// Change the position of a control point
	if(i<0 || i>=m_size[0] || j<0 || j>=m_size[1]) return;
	m_points[j][i].pos[0] = pos[0];
	m_points[j][i].pos[1] = pos[1];
}


void Grid::getClosestControlPoint(float pos[2], int &i, int &j) const
{
	// Iterate through each point and check which one is the closest
	float minSqDist = 1e10;
	for(int jj=0; jj<m_size[1]; ++jj)
		for(int ii=0; ii<m_size[0]; ++ii)
	{
		// Compute the distance between current control point
		// and given position
		float diff[2] = {
			m_points[jj][ii].pos[0] - pos[0],
			m_points[jj][ii].pos[1] - pos[1]
		};
		float sqDist = (diff[0]*diff[0])+(diff[1]*diff[1]);
		
		// Check if it is the closest point so far
		if(sqDist < minSqDist)
		{
			i = ii;
			j = jj;
			minSqDist = sqDist;
		}
	}
}
