/* image
 * $AUTHORS$, Tue Jun 6 2006
 *
 */
#include "image.h"
#include <fstream>
#include <iostream>

Image::Image(int width, int height)
:	m_data(0)
{
	setSize(width, height);
}


Image::~Image()
{
	// Destroy data memory
	if(m_data) delete [] m_data;
}


void Image::setSize(int width, int height)
{
	// Set the new image size
	m_size[0] = width;
	m_size[1] = height;
	
	// Delete any previous memory
	if(m_data) delete [] m_data;
	m_data = 0;
	
	// If a dimension is 0, leave data uninitialized
	if(!m_size[0] || !m_size[1]) return;
	
	// Allocate memory
	m_data = new unsigned char [width*height];
	
	// Initialize it with 0s
	for(int i=0; i<width*height; ++i)
		m_data[i] = 0;
}


const int* Image::getSize() const
{
	// Return the image size
	return m_size;
}


unsigned char Image::get(int i, int j) const
{
	// If wrong coordinate, just return black
	if(!m_data || i<0 || i>=m_size[0] || j<0 || j>=m_size[1])
		return 0;

	// Return the intensity at pixel (i,j)
	int index = (j*m_size[0])+i;
	return m_data[index];
}

void Image::set(int i, int j, unsigned char val)
{
	// If wrong coordinate, just do nothing
	if(!m_data || i<0 || i>=m_size[0] || j<0 || j>=m_size[1])
		return;

	// Set intensity at pixel (i,j)
	int index = (j*m_size[0])+i;
	m_data[index] = val;
}
