/* imageio
 * $AUTHORS$, Tue Jun 6 2006
 *
 */
#ifndef IMAGEIO_H
#define IMAGEIO_H

/**
	@author Herve Lombaert <herve.lombaert@polymtl.ca>
*/
template <typename Image>
class ImageIO{
public:
	ImageIO(Image* image);
	~ImageIO();

	bool readPGM(const char* filename);
	bool writePGM(const char* filename) const;
	
private:
	Image* m_image;
};


#include <fstream>
#include <cstdio>

template <typename Image>
ImageIO<Image>::ImageIO(Image* image)
:	m_image(image)
{}


template <typename Image>
ImageIO<Image>::~ImageIO()
{}


template <typename Image>
bool ImageIO<Image>::readPGM(const char* filename)
{
	using namespace std;

	string buf;
	char c_buf[1024];
	ifstream file;
	int width, height;

	file.open(filename);
	if(file.fail()) {
		printf("Error while opening: %s\n", filename);
		return 1;
	}

	file >> buf;
	if(buf != "P2") {
		printf("Error while reading: %s\n", filename);
		return 1;
	}

	file.ignore(1);
	file.getline(&c_buf[0], 1024); // ignore comment, assume there is a comment

	file >> width >> height;
	file >> buf;
	if(buf != "255") {
		printf("Format not supported\n");
		return 1;
	}
	
	m_image->setSize(width, height);

	for(int h=height-1; h>=0; --h)
		for(int w=0; w<width; ++w)
	{
		file >> buf;
		m_image->set(w,h,atoi(buf.c_str()));
	}

	return 0;
}


template <typename Image>
bool ImageIO<Image>::writePGM(const char* filename) const
{
	using namespace std;

	ofstream file;
	file.open(filename, ios::binary);
	if(file.fail()) {
		printf("Error while creating: %s\n", filename);
		return 1;
	}

	file << "P5" << endl; // writing binary format, easier
	file << "# No Comment" << endl;
	file << m_image->getSize()[0] << " " << m_image->getSize()[1] << endl;
	file << "255" << endl;

	for(int h=m_image->getSize()[1]-1; h>=0; h--) {
		for(int w=0; w<m_image->getSize()[0]; w++) {
			unsigned char val = m_image->get(w,h);
			file.write((char*)&val, 1);
		}
	}

	file.close();

	return 0;
}

#endif
